#needsrootforbuild
# Build documentation package
%bcond_with doc

Name: spdk
Version: 24.01
Release: 1%{?dist}
Summary: Set of libraries and utilities for high performance user-mode storage
License: BSD and MIT
URL: http://spdk.io
Source0: https://github.com/spdk/spdk/archive/refs/tags/v%{version}.tar.gz
Patch1: 0001-Add-without-ISA-L-option-and-disabled-by-default.patch
Patch2: 0002-backport-Add-ctrlr_lock-for-cuse-register-and-unregister.patch
Patch3: 0003-add-HSAK-needed-head-file-and-API-to-spdk.patch
Patch4: 0004-lib-bdev-Add-bdev-support-for-HSAK.patch
Patch5: 0005-lib-env_dpdk-Add-config-args-for-HSAK.patch
Patch6: 0006-lib-nvme-Add-nvme-support-for-HSAK.patch
Patch7: 0007-module-bdev-Add-bdev-module-support-for-HSAK.patch
Patch8: 0008-use-spdk_nvme_ns_cmd_dataset_management-and-delete-s.patch
Patch9: 0009-spdk-add-nvme-support-for-HSAK.patch
Patch10: 0010-Add-CUSE-switch-for-nvme-ctrlr.patch
Patch11: 0011-Adapt-for-ES3000-serial-vendor-special-opcode-in-CUS.patch
Patch12: 0012-adapt-for-spdk-24.01.patch
Patch13: 0013-modify-to-pip3.patch

%define package_version %{version}-%{release}

%define install_datadir %{buildroot}/%{_datadir}/%{name}
%define install_sbindir %{buildroot}/%{_sbindir}
%define install_docdir %{buildroot}/%{_docdir}/%{name}

# Distros that don't support python3 will use python2
%if "%{dist}" == ".el7"
%define use_python2 1
%else
%define use_python2 0
%endif

ExclusiveArch: x86_64 aarch64 loongarch64 ppc64le riscv64

BuildRequires: gcc gcc-c++ make
BuildRequires: dpdk-devel >= 23.11, numactl-devel, ncurses-devel
BuildRequires: libiscsi-devel, libaio-devel, openssl-devel, libuuid-devel
BuildRequires: libibverbs-devel, librdmacm-devel
BuildRequires: fuse3, fuse3-devel
BuildRequires: libboundscheck
BuildRequires: patchelf
BuildRequires: CUnit, CUnit-devel, python3-pip, chrpath
%if %{with doc}
BuildRequires: doxygen mscgen graphviz
%endif

# Install dependencies
Requires: dpdk >= 23.11, numactl-libs, openssl-libs
Requires: libiscsi, libaio, libuuid
Requires: fuse3, libboundscheck
# NVMe over Fabrics
Requires: librdmacm, librdmacm
Requires(post): /sbin/ldconfig
Requires(postun): /sbin/ldconfig

%description
The Storage Performance Development Kit provides a set of tools
and libraries for writing high performance, scalable, user-mode storage
applications.


%package devel
Summary: Storage Performance Development Kit development files
Requires: %{name}%{?_isa} = %{package_version}
Provides: %{name}-static%{?_isa} = %{package_version}

%description devel
This package contains the headers and other files needed for
developing applications with the Storage Performance Development Kit.


%package tools
Summary: Storage Performance Development Kit tools files
%if "%{use_python2}" == "0"
Requires: %{name} = %{package_version} python3 python3-configshell python3-pexpect
%else
Requires: %{name} = %{package_version} python python-configshell pexpect
%endif
BuildArch: noarch

%description tools
%{summary}


%if %{with doc}
%package doc
Summary: Storage Performance Development Kit documentation
BuildArch: noarch

%description doc
%{summary}
%endif


%prep
# add -q
%autosetup -n spdk-%{version} -p1

%build
./configure --prefix=%{_usr} \
	--disable-tests \
	--disable-examples \
	--disable-unit-tests \
	--disable-apps \
	--without-crypto \
	--without-isal \
	--with-dpdk \
	--without-fio \
	--with-vhost \
	--without-rbd \
	--with-rdma \
	--with-shared \
	--with-iscsi-initiator \
	--without-vtune \
	--enable-raw \
	--with-nvme-cuse \

make -j`nproc` all

%if %{with doc}
make -C doc
%endif

%install
%make_install -j`nproc` prefix=%{_usr} libdir=%{_libdir} datadir=%{_datadir}
install -d $RPM_BUILD_ROOT%{_sysconfdir}/spdk
install -d $RPM_BUILD_ROOT/opt/spdk
install -d $RPM_BUILD_ROOT/usr/include/spdk_internal
install -m 0744 ./scripts/setup_self.sh $RPM_BUILD_ROOT/opt/spdk/setup.sh
install -m 0644 ./etc/spdk/nvme.conf.in $RPM_BUILD_ROOT%{_sysconfdir}/spdk
install -m 0644 include/spdk_internal/*.h $RPM_BUILD_ROOT/usr/include/spdk_internal
install -m 0644 lib/nvme/nvme_internal.h $RPM_BUILD_ROOT/usr/include/spdk_internal
install -m 0644 lib/env_dpdk/22.11/*.h $RPM_BUILD_ROOT/usr/include/spdk

# Install tools
mkdir -p %{install_datadir}
find scripts -type f -regextype egrep -regex '.*(spdkcli|rpc).*[.]py' \
	-exec cp --parents -t %{install_datadir} {} ";"

# env is banned - replace '/usr/bin/env anything' with '/usr/bin/anything'
find %{install_datadir}/scripts -type f -regextype egrep -regex '.*([.]py|[.]sh)' \
	-exec sed -i -E '1s@#!/usr/bin/env (.*)@#!/usr/bin/\1@' {} +

%if "%{use_python2}" == "1"
find %{install_datadir}/scripts -type f -regextype egrep -regex '.*([.]py)' \
	-exec sed -i -E '1s@#!/usr/bin/python3@#!/usr/bin/python2@' {} +
%endif

# synlinks to tools
mkdir -p %{install_sbindir}
ln -sf -r %{install_datadir}/scripts/rpc.py %{install_sbindir}/%{name}-rpc
ln -sf -r %{install_datadir}/scripts/spdkcli.py %{install_sbindir}/%{name}-cli

%if %{with doc}
# Install doc
mkdir -p %{install_docdir}
mv doc/output/html/ %{install_docdir}
%endif


%files
%dir /opt/spdk
/opt/spdk/setup.sh
%dir %{_sysconfdir}/spdk
%{_sysconfdir}/spdk/nvme.conf.in
/usr/lib/python%{python3_version}/site-packages/spdk*/*
%{_bindir}/spdk_*
%{_libdir}/*.so.*


%files devel
%{_libdir}/pkgconfig/*.pc
%{_includedir}/%{name}
%{_libdir}/*.a
%{_libdir}/*.so
%dir /usr/include/spdk_internal
/usr/include/spdk_internal/*.h


%files tools
%{_datadir}/%{name}/scripts
%{_sbindir}/%{name}-rpc
%{_sbindir}/%{name}-cli

%if %{with doc}
%files doc
%{_docdir}/%{name}
%endif


%changelog
* Mon Mar 10 2025 Kaiqiang Wang <wangkaiqiang@ieisystem.com> - 24.01-1
- init to 24.01
